@@ -197,14 +197,6 @@ struct bbm1_cp
       /* relative tolerance which is used to check if current value of
          the objective function is not better than for the best integer
          feasible solution found */
-      int form;
-      /* this parameter specifies what form of the basis matrix should
-         be used by the solver:
-         0 - PFI
-         1 - RFI + Bartels & Golub updating technique
-         2 - RFI + Forrest & Tomlin updating technique
-         3 - AFI
-         4 - UFI */
       int steep;
       /* if this flag is set, the solver uses the steepest edge pricing
          proposed by Goldfarb & Reid; otherwise the standard "textbook"
