@@ -26,6 +26,8 @@
 #ifndef _GLPK_H
 #define _GLPK_H
 
+#include "glplpx.h"
+
 #ifndef _GLPAPI_H
 typedef struct { int dummy; } LPI;
 #endif
@@ -206,9 +208,12 @@ extern void glp_set_row_fctr(LPI *lp, int i, double fctr);
 extern void glp_set_row_name(LPI *lp, int i, char *name);
 /* assign (change) row name */
 
-extern void *glp_to_lpx(LPI *lpi, int all);
+extern LPX *glp_lpi_to_lpx(LPI *lpi, int all);
 /* convert LP problem instance to low-level object */
 
+extern void glp_load_lpx_soln(LPI *lpi, LPX *lpx, int round);
+/* load LP solution from low-level object */
+
 struct wmps
 {     /* parameter block of the routine glp_write_mps() */
       int prob_info;
@@ -245,64 +250,75 @@ extern void glp_init_wmps(struct wmps *parm);
 extern int glp_write_mps(LPI *lp, char *fname, struct wmps *parm);
 /* write problem data using MPS format */
 
-struct rsm1
-{     /* revised simplex method parameter block */
-      int what;
-      /* this parameter specifies what basis solution should be found
-         by the solver:
-         0 - initial solution
-         1 - primal feasible solution
-         2 - optimal solution */
+struct spx
+{     /* simplex method parameter block */
+      int msg_lev;
+      /* level of messages output by the solver:
+         0 - no output
+         1 - error messages only
+         2 - normal output
+         3 - full output (includes informational messages) */
       int scale;
-      /* if this flag is set, the solver performs automatic scaling the
-         problem; otherwise the scaling is not used */
-      int dual;
-      /* if this flag is set, the solver uses the dual simplex method
-         in order to find the primal feasible basis solution; otherwise
-         the solver uses the primal simplex method */
-      int steep;
-      /* if this flag is set, the solver uses the steepest edge pricing
-         (for both primal and dual simplex) proposed by Goldfarb & Reid;
-         otherwise the solver uses the standard "textbook" pricing */
+      /* scaling option:
+         0 - do not scale the problem
+         1 - scale the problem before solving it */
+      int initb;
+      /* initial basis option:
+         0 - use the standard basis, where all auxiliary variables are
+             basic and all structural variables are non-basic
+         1 - use an advanced initial basis
+         2 - use the basis specified in the problem object */
+      int price;
+      /* pricing option:
+         0 - use standard ("textbook") pricing
+         1 - use projected steepest edge pricing */
       int relax;
-      /* if this flag is set, the solver uses two-pass ratio test (for
-         both primal and dual simplex) proposed by P.Harris; otherwise
-         the solver uses the standard "textbook" ratio test */
+      /* ratio test option:
+         0 - use the standard ("textbook") ratio test
+         1 - use two-pass ratio test proposed by P.Harris */
       double tol_bnd;
-      /* relative tolerance which is used to see if the solution is
-         primal feasible */
+      /* relative tolerance used to check if the current basic solution
+         is primal feasible */
       double tol_dj;
-      /* relative tolerance which is used to see if the solution is
-         dual feasible */
+      /* absolute tolerance used to check if the current basic solution
+         is dual feasible */
       double tol_piv;
-      /* relative tolerance which is used to choose the pivot element
-         of the simplex table */
-      int iter_max;
-      /* maximal allowed number of iterations (0 means no limit) */
+      /* relative tolerance used to choose eligible pivotal elements of
+         the simplex table in the ratio test */
       int round;
-      /* if this flag is set, the solver replaces computed values of
-         basic variables and reduced costs of non-basic variables which
-         are close to zero by exact zeros; otherwise the solver remains
-         all computed values and reduced costs "as is" */
+      /* solution rounding option:
+         0 - report all computed values and reduced costs "as is"
+         1 - round computed values of basic variables and reduced costs
+             of non-basic variables toward their bounds or toward zero,
+             if possible (i.e. if it is allowed by the tolerances) */
+      int it_lim;
+      /* simplex iterations limit; if this value is positive, it is
+         decreased by one each time when one simplex iteration has been
+         performed, and reaching zero value signals the solver to stop
+         the search; negative value means no iterations limit */
+      double tm_lim;
+      /* searching time limit, in seconds; if this value is positive,
+         it is decreased each time when one simplex iteration has been
+         performed by the amount of time spent for the iteration, and
+         reaching zero value signals the solver to stop the search;
+         negative value means no time limit */
+      int out_frq;
+      /* output frequency, in iterations; this parameter specifies how
+         frequently the solver sends information about the solution to
+         the standard output */
 };
 
-extern void glp_init_rsm1(struct rsm1 *parm);
+extern void glp_init_spx(struct spx *parm);
 /* initialize parameter block by default values */
 
-extern int glp_call_rsm1(LPI *lpi, struct rsm1 *parm);
-/* solve LP problem using revised simplex method */
+extern int glp_simplex(LPI *lp, struct spx *parm);
+/* solve LP problem using simplex method */
 
-extern int glp_call_ipm1(LPI *lpi);
+extern int glp_interior(LPI *lpi);
 /* solve LP problem using interior point method */
 
-struct bbm1
+struct bnb
 {     /* branch-and-bound method parameter block */
-      int what;
-      /* this parameter specifies what basis solution should be found
-         by the solver:
-         0 - initial relaxed solution
-         1 - feasible integer solution
-         2 - optimal integer solution */
       int branch;
       /* this parameter specifies what branching heuristic should be
          used by the solver: */
@@ -322,19 +338,9 @@ struct bbm1
       /* relative tolerance which is used to check if current value of
          the objective function is not better than for the best integer
          feasible solution found */
-      int meth;
-      /* this parameter specifies what version of the simplex method
-         should be used by the solver to find an optimal basis of LP
-         relaxation:
-         0 - rsm1_driver()
-         1 - glp_simplex2() */
       int scale;
       /* if this flag is set, the solver performs automatic scaling the
          problem; otherwise the scaling is not used */
-      int dual;
-      /* if this flag is set, the solver uses the dual simplex method
-         in order to find the primal feasible basis solution; otherwise
-         the solver uses the primal simplex method */
       int steep;
       /* if this flag is set, the solver uses the steepest edge pricing
          proposed by Goldfarb & Reid; otherwise the standard "textbook"
@@ -357,162 +363,15 @@ struct bbm1
       int round;
       /* if this flag is set, the solver replaces computed values of
          basic continuous variables which are close to zero by exact
-         zeros; otherwise all computed values are remained "as is" */
+         zeros; otherwise all computed values remain "as is" */
 };
 
-extern void glp_init_bbm1(struct bbm1 *parm);
+extern void glp_init_bnb(struct bnb *parm);
 /* initialize parameter block by default values */
 
-extern int glp_call_bbm1(LPI *lpi, struct bbm1 *parm);
+extern int glp_integer(LPI *lpi, struct bnb *parm);
 /* solve MIP problem using branch-and-bound method */
 
-struct spx1
-{     /* comprehensive simplex driver parameter block */
-      int scale;
-      /* this parameter controls the scaling option:
-         0 - do not scale the problem
-         1 - scale the problem using scale factors specified in the
-             problem object */
-      int initb;
-      /* this parameter controls the initial basis option:
-         0 - use the standard basis, where all auxiliary variables are
-             basic and all structural variables are non-basic
-         1 - use the basis specified in the problem object */
-      int feas;
-      /* this parameter specifies by what method the solver should
-         search for primal feasible solution if the initial basis is
-         not primal feasible:
-         0 - use the primal simplex method and implicit artificial
-             variables technique
-         1 - use the dual simplex method and artificial objective
-             function technique */
-      int dual;
-      /* this parameter specifies what the solver should undertake if
-         the initial basis happens to be dual feasible:
-         0 - ignore dual feasibility, i.e. search for primal feasible
-             solution (phase I) and then search for optimal solution
-             (phase II) using the primal simplex method
-         1 - search for optimal solution using the dual simplex method,
-             i.e. skip the phase I */
-      int steep;
-      /* this parameter controls the pricing option (for both primal
-         and dual simplex methods):
-         0 - use the standard ("textbook") pricing
-         1 - use the steepest edge pricing proposed by Goldfarb and
-             Reid */
-      int relax;
-      /* this parameter controls the ratio test option (for both primal
-         and dual simplex methods):
-         0 - use the standard ("textbook") ratio test
-         1 - use two-pass ratio test proposed by P.Harris */
-      int round;
-      /* this parameter controls the final solution rounding option:
-         0 - keep all computed primal and dual values "as is"
-         1 - replace computed primal and dual values, which are close
-             to zero, by exact zeros (this is performed once after the
-             final solution has been obtained) */
-      double tol_bnd;
-      /* relative tolerance which is used to see if the solution is
-         primal feasible */
-      double tol_dj;
-      /* relative tolerance which is used to see if the solution is
-         dual feasible */
-      double tol_piv;
-      /* relative tolerance which is used to choose the pivot element
-         of the simplex table */
-      int max_iter;
-      /* maximal allowable number of simplex iterations; if this
-         parameter is zero, the solver performs no simplex iterations,
-         but reports solution that corresponds to the initial basis */
-      double max_time;
-      /* maximal allowable time of computations, in seconds; if this
-         parameter is zero, the solver performs no simplex iterations,
-         but reports solution that corresponds to the initial basis */
-      double obj_min, obj_max;
-      /* allowable range of the objective function; the solver stops
-         searching as soon as on the phase II the current value of the
-         objective function becomes less than obj_min and continue
-         decreasing or greater than obj_max and continue increasing;
-         to avoid computing the objective function on each simplex
-         iteration, the range (-DBL_MAX, +DBL_MAX) should be used */
-      int quiet;
-      /* this parameter controls displaying informative messages:
-         0 - display all informative messages
-         1 - don't display any informative messages */
-      double delay;
-      /* initial delay interval, in seconds; the solver doesn't display
-         information about current solution the first delay seconds */
-      double freq;
-      /* output frequency, in seconds; the solver displays information
-         about current solution approx. once per freq seconds */
-      int iter_used;
-      /* on entry this parameter is ignored; on exit this parameter is
-         number of actually performed simplex iterations */
-      double time_used;
-      /* on entry this parameter is ignored; on exit this parameter is
-         elapsed time (in seconds) actually spent by the solver */
-};
-
-extern void glp_init_spx1(struct spx1 *parm);
-/* initialize parameter block by default values */
-
-extern int glp_simplex1(LPI *lp, struct spx1 *_parm);
-/* comprehensive driver to the simplex method */
-
-struct piv1
-{     /* pivoting simplex driver parameter block */
-      int scale;
-      /* this parameter controls the scaling option:
-         0 - do not scale the constraint matrix
-         1 - scale the constraint matrix using scale factors specified
-             in the problem object */
-      double tol;
-      /* Markowitz tolerance */
-};
-
-extern void glp_init_piv1(struct piv1 *parm);
-/* initialize parameter block by default values */
-
-extern int glp_pivot_in(LPI *lp, int flag[], struct piv1 *parm);
-/* push auxiliary variables into the basis */
-
-extern int glp_pivot_out(LPI *lp, int flag[], struct piv1 *parm);
-/* pull structural variables from the basis */
-
-struct spx2
-{     /* primal simplex method parameter block */
-      int scale;
-      /* this parameter controls the scaling option:
-         0 - do not scale the problem
-         1 - scale the problem
-         2 - scale the problem using scale factors specified in the
-             problem object */
-      int initb;
-      /* this parameter controls the initial basis option:
-         0 - use the standard basis, where all auxiliary variables are
-             basic and all structural variables are non-basic
-         1 - use an advanced initial basis
-         2 - use the basis specified in the problem object */
-      int round;
-      /* this parameter controls the final solution rounding option:
-         0 - keep all computed primal and dual values "as is"
-         1 - replace computed primal and dual values, which are close
-             to zero, by exact zeros (this is performed once after the
-             final solution has been obtained) */
-      double tol_bnd;
-      /* tolerance used to check if the solution is primal feasible */
-      double tol_dj;
-      /* tolerance used to check if the solution is dual feasible */
-      double tol_piv;
-      /* tolerance used to choose pivot element of the simplex table */
-};
-
-extern void glp_init_spx2(struct spx2 *parm);
-/* initialize parameter block by default values */
-
-extern int glp_simplex2(LPI *lp, struct spx2 *parm);
-/* solve LP problem using primal simplex method */
-
 #endif
 
 /* eof */
