@@ -74,6 +74,8 @@ struct ENV
       /* total number of currently allocated memory blocks */
       int mem_cpeak;
       /* peak value of mem_count */
+      double last_watch;
+      /* the value returned by the routine watch on the last call */
 };
 
 extern int init_lib_env(void);
