@@ -27,10 +27,10 @@
 #define _GLPLPX_H
 
 #include "glpinv.h"
+#include "glpspm.h"
 #include "glpstr.h"
 
 #define lpx_create_prob       _glp_lpx_create_prob
-#define lpx_reset_parms       _glp_lpx_reset_parms
 #define lpx_realloc_prob      _glp_lpx_realloc_prob
 #define lpx_add_rows          _glp_lpx_add_rows
 #define lpx_add_cols          _glp_lpx_add_cols
@@ -40,22 +40,23 @@
 #define lpx_set_col_name      _glp_lpx_set_col_name
 #define lpx_set_row_bnds      _glp_lpx_set_row_bnds
 #define lpx_set_col_bnds      _glp_lpx_set_col_bnds
+#define lpx_set_col_kind      _glp_lpx_set_col_kind
 #define lpx_set_obj_dir       _glp_lpx_set_obj_dir
 #define lpx_set_obj_c0        _glp_lpx_set_obj_c0
 #define lpx_set_row_coef      _glp_lpx_set_row_coef
 #define lpx_set_col_coef      _glp_lpx_set_col_coef
-#define lpx_defrag_sva        _glp_lpx_defrag_sva
-#define lpx_enlarge_cap       _glp_lpx_enlarge_cap
+#define lpx_set_row_stat      _glp_lpx_set_row_stat
+#define lpx_set_col_stat      _glp_lpx_set_col_stat
+#define lpx_load_mat          _glp_lpx_load_mat
 #define lpx_set_mat_row       _glp_lpx_set_mat_row
 #define lpx_set_mat_col       _glp_lpx_set_mat_col
 #define lpx_unmark_all        _glp_lpx_unmark_all
 #define lpx_mark_row          _glp_lpx_mark_row
 #define lpx_mark_col          _glp_lpx_mark_col
-#define lpx_change_ordn       _glp_lpx_change_ordn
 #define lpx_clear_mat         _glp_lpx_clear_mat
 #define lpx_del_items         _glp_lpx_del_items
 #define lpx_delete_prob       _glp_lpx_delete_prob
-#define lpx_simplex           _glp_lpx_simplex
+
 #define lpx_get_num_rows      _glp_lpx_get_num_rows
 #define lpx_get_num_cols      _glp_lpx_get_num_cols
 #define lpx_get_num_nz        _glp_lpx_get_num_nz
@@ -64,14 +65,41 @@
 #define lpx_get_col_name      _glp_lpx_get_col_name
 #define lpx_get_row_bnds      _glp_lpx_get_row_bnds
 #define lpx_get_col_bnds      _glp_lpx_get_col_bnds
+#define lpx_get_col_kind      _glp_lpx_get_col_kind
 #define lpx_get_obj_dir       _glp_lpx_get_obj_dir
 #define lpx_get_obj_c0        _glp_lpx_get_obj_c0
 #define lpx_get_row_coef      _glp_lpx_get_row_coef
 #define lpx_get_col_coef      _glp_lpx_get_col_coef
 #define lpx_get_mat_row       _glp_lpx_get_mat_row
 #define lpx_get_mat_col       _glp_lpx_get_mat_col
+#define lpx_get_status        _glp_lpx_get_status
 #define lpx_get_prim_stat     _glp_lpx_get_prim_stat
 #define lpx_get_dual_stat     _glp_lpx_get_dual_stat
+#define lpx_get_row_info      _glp_lpx_get_row_info
+#define lpx_get_col_info      _glp_lpx_get_col_info
+#define lpx_get_obj_val       _glp_lpx_get_obj_val
+#define lpx_print_sol         _glp_lpx_print_sol
+
+#define lpx_reset_parms       _glp_lpx_reset_parms
+#define lpx_set_int_parm      _glp_lpx_set_int_parm
+#define lpx_get_int_parm      _glp_lpx_get_int_parm
+#define lpx_set_real_parm     _glp_lpx_set_real_parm
+#define lpx_get_real_parm     _glp_lpx_get_real_parm
+
+#define lpx_scale_prob        _glp_lpx_scale_prob
+#define lpx_unscale_prob      _glp_lpx_unscale_prob
+
+#define lpx_std_basis         _glp_lpx_std_basis
+#define lpx_adv_basis         _glp_lpx_adv_basis
+
+#define lpx_warm_up           _glp_lpx_warm_up
+#define lpx_prim_opt          _glp_lpx_prim_opt
+#define lpx_prim_art          _glp_lpx_prim_art
+#define lpx_dual_opt          _glp_lpx_dual_opt
+#define lpx_simplex           _glp_lpx_simplex
+
+#define lpx_eval_tab_row      _glp_lpx_eval_tab_row
+#define lpx_eval_tab_col      _glp_lpx_eval_tab_col
 
 /*----------------------------------------------------------------------
 -- The structure LPX defines a low-level LP problem object and includes
@@ -185,7 +213,8 @@
 --
 --    pi = (pi[i]) = inv(B') * cB                                   (16)
 --
--- is the vector of simplex (Lagrange) multipliers.
+-- is the vector of simplex (Lagrange) multipliers, which correspond to
+-- the equiality constraints (5).
 --
 -- Note that signs of the reduced costs d are determined by the formula
 -- (15) in both cases of minimization and maximization.
@@ -231,14 +260,17 @@
 
 typedef struct LPX LPX;
 
+/* CAUTION: DO NOT CHANGE ANY MEMBERS OF THIS STRUCTURE DIRECTLY; THIS
+   MAY DAMAGE ITS INTEGRITY AND CAUSE UNPREDICTABLE ERRORS! */
+
 struct LPX
 {     /* LP problem object */
       int m_max;
-      /* maximal number of rows (if necessary, this parameter is
-         automatically increased) */
+      /* maximal number of rows (if necessary, this parameter can be
+         automatically changed) */
       int n_max;
-      /* maximal number of columns (if necessary, this parameter is
-         automatically increased) */
+      /* maximal number of columns (if necessary, this parameter can be
+         automatically changed) */
       int m;
       /* current number of rows (auxiliary variables) */
       int n;
@@ -248,18 +280,18 @@ struct LPX
       char *buf; /* char buf[255+1]; */
       /* working buffer used to return character strings */
       /*--------------------------------------------------------------*/
-      /* symbolic names, types, and bounds of variables */
+      /* LP problem data */
       STR **name; /* STR *name[1+m+n]; */
       /* name[0] is a name of the LP problem object;
          name[k], 1 <= k <= m+n, is a name of the variable x[k] */
       int *typx; /* int typx[1+m_max+n_max]; */
       /* typx[0] is not used;
          typx[k], 1 <= k <= m+n, is the type of the variable x[k]: */
-#define LPX_FR          10 /* free variable:    -inf <  x[k] < +inf  */
-#define LPX_LO          11 /* lower bound:      l[k] <= x[k] < +inf  */
-#define LPX_UP          12 /* upper bound:      -inf <  x[k] <= u[k] */
-#define LPX_DB          13 /* double bound:     l[k] <= x[k] <= u[k] */
-#define LPX_FX          14 /* fixed variable:   l[k]  = x[k]  = u[k] */
+#define LPX_FR          100   /* free variable:  -inf <  x[k] < +inf  */
+#define LPX_LO          101   /* lower bound:    l[k] <= x[k] < +inf  */
+#define LPX_UP          102   /* upper bound:    -inf <  x[k] <= u[k] */
+#define LPX_DB          103   /* double bound:   l[k] <= x[k] <= u[k] */
+#define LPX_FX          104   /* fixed variable: l[k]  = x[k]  = u[k] */
       double *lb; /* double lb[1+m_max+n_max]; */
       /* lb[0] is not used;
          lb[k], 1 <= k <= m+n, is an lower bound of the variable x[k];
@@ -280,111 +312,45 @@ struct LPX
          mark[k], 1 <= k <= m+n, is a marker of the variable x[k];
          these markers are used to mark rows and columns for various
          purposes */
-      /*--------------------------------------------------------------*/
-      /* objective function */
       int dir;
       /* optimization direction (sense of the objective function): */
-#define LPX_MIN         20 /* minimization */
-#define LPX_MAX         21 /* maximization */
+#define LPX_MIN         110   /* minimization */
+#define LPX_MAX         111   /* maximization */
       double *coef; /* double coef[1+m_max+n_max]; */
       /* coef[0] is a constant term of the objective function;
          coef[k], 1 <= k <= m+n, is a coefficient of the objective
          function at the variable x[k] (note that auxiliary variables
          also may have non-zero objective coefficients) */
+      SPM *A; /* SPM A[1:m,1:n]; */
+      /* constraint matrix (has m rows and n columns) */
       /*--------------------------------------------------------------*/
-      /* constraint matrix in row/column-wise sparse format */
-      int *aa_ptr; /* int aa_ptr[1+m_max+n_max]; */
-      /* aa_ptr[0] is not used;
-         aa_ptr[i], 1 <= i <= m, is a pointer to the first element of
-         the i-th row in the sparse vector area;
-         aa_ptr[m+j], 1 <= j <= n, is a pointer to the first element of
-         the j-th column in the sparse vector area */
-      int *aa_len; /* int aa_len[1+m_max+n_max]; */
-      /* aa_len[0] is not used;
-         aa_len[i], 1 <= i <= m, is number of (non-zero) elements in
-         the i-th row, 0 <= aa_len[i] <= n;
-         aa_len[m+j], 1 <= j <= n, is number of (non-zero) elements in
-         the j-th column, 0 <= aa_len[m+j] <= m */
-      int *aa_cap; /* int aa_cap[1+m_max+n_max]; */
-      /* aa_cap[0] is not used;
-         aa_cap[i], 1 <= i <= m, is capacity of the i-th row that is
-         number of locations, which can be used without relocation the
-         row, aa_cap[i] >= aa_len[i];
-         aa_cap[m+j], 1 <= j <= n, is capacity of the j-th column that
-         is number of locations, which can be used without relocation
-         the column, aa_cap[m+j] >= aa_len[m+j] */
-      /*--------------------------------------------------------------*/
-      /* sparse vector area (SVA) is a set of locations intended to
-         store sparse vectors that represent rows and columns of the
-         constraint matrix; each location is a doublet (ndx, val),
-         where ndx is an index and val is a numerical value of sparse
-         vector element; in the whole each sparse vector is a set of
-         adjacent locations defined by a pointer to the first element
-         (aa_ptr[*]) and number of its non-zero elements (aa_len[*]) */
-      int sv_size;
-      /* total size of SVA, in locations; locations are numbered by
-         integers 1, 2, ..., sv_size, and location with the number 0 is
-         not used; if necessary, sv_size is automatically increased */
-      int sv_used;
-      /* locations with the numbers 1, 2, ..., sv_used are currently
-         used by rows and columns of the constraint matrix (this extent
-         may contain "holes" due to fragmentation);
-         locations with the numbers sv_used+1, sv_used+2, ..., sv_size
-         are currently free;
-         total number of free locations is (sv_size - sv_used) */
-      int *sv_ndx; /* int sv_ndx[1+sv_size]; */
-      /* sv_ndx[0] is not used;
-         sv_ndx[k], 1 <= k <= sv_size, is the index field of the k-th
-         location */
-      double *sv_val; /* double sv_val[1+sv_size]; */
-      /* sv_val[0] is not used;
-         sv_val[k], 1 <= k <= sv_size, is the value field of the k-th
-         location */
-      /*--------------------------------------------------------------*/
-      /* in order to efficiently defragment SVA a doubly linked list of
-         rows and columns of the constraint matrix is used, where rows
-         have numbers 1, ..., m, and column have numbers m+1, ..., m+n,
-         so each row and column can be uniquely identified by only one
-         integer; in the list rows and columns are ordered by ascending
-         their pointers aa_ptr[*] */
-      int sv_head;
-      /* the number of the leftmost (in SVA) row/column */
-      int sv_tail;
-      /* the number of the rightmost (in SVA) row/column */
-      int *sv_prev; /* int sv_prev[1+m_max+n_max]; */
-      /* sv_prev[k], 1 <= k <= m+n, is the number of a row/column that
-         precedes (in SVA) the k-th row/column */
-      int *sv_next; /* int sv_next[1+m_max+n_max]; */
-      /* sv_next[k], 1 <= k <= m+n, is the number of a row/column that
-         succedes (in SVA) the k-th row/column */
-      /*--------------------------------------------------------------*/
-      /* current basis */
+      /* current basis and basic solution components */
       int b_stat;
       /* status of the current basis: */
-#define LPX_B_UNDEF     30 /* the current basis is undefined */
-#define LPX_B_VALID     31 /* the current basis is valid */
+#define LPX_B_UNDEF     120   /* the current basis is undefined */
+#define LPX_B_VALID     121   /* the current basis is valid */
       int p_stat;
       /* status of the primal solution: */
-#define LPX_P_UNDEF     40 /* the status is undefined */
-#define LPX_P_FEAS      41 /* the solution is primal feasible */
-#define LPX_P_INFEAS    42 /* the solution is primal infeasible */
-#define LPX_P_NOFEAS    43 /* no primal feasible solution exists */
+#define LPX_P_UNDEF     130   /* the status is undefined */
+#define LPX_P_FEAS      131   /* the solution is primal feasible */
+#define LPX_P_INFEAS    132   /* the solution is primal infeasible */
+#define LPX_P_NOFEAS    133   /* no primal feasible solution exists */
       int d_stat;
       /* status of the dual solution: */
-#define LPX_D_UNDEF     50 /* the status is undefined */
-#define LPX_D_FEAS      51 /* the solution is dual feasible */
-#define LPX_D_INFEAS    52 /* the solution is dual infeasible */
-#define LPX_D_NOFEAS    53 /* no dual feasible solution exists */
+#define LPX_D_UNDEF     140   /* the status is undefined */
+#define LPX_D_FEAS      141   /* the solution is dual feasible */
+#define LPX_D_INFEAS    142   /* the solution is dual infeasible */
+#define LPX_D_NOFEAS    143   /* no dual feasible solution exists */
       int *tagx; /* int tagx[1+m_max+n_max]; */
       /* tagx[0] is not used;
          tagx[k], 1 <= k <= m+n, is the status of the variable x[k]
          (contents of this array is always defined independently on the
          status of the current basis): */
-#define LPX_BS          60 /* basic variable */
-#define LPX_NL          61 /* non-basic variable on its lower bound */
-#define LPX_NU          62 /* non-basic variable on its upper bound */
-#define LPX_NF          63 /* non-basic free variable */
-#define LPX_NS          64 /* non-basic fixed variable */
+#define LPX_BS          150   /* basic variable */
+#define LPX_NL          151   /* non-basic variable on lower bound */
+#define LPX_NU          152   /* non-basic variable on upper bound */
+#define LPX_NF          153   /* non-basic free variable */
+#define LPX_NS          154   /* non-basic fixed variable */
       int *posx; /* int posx[1+m_max+n_max]; */
       /* posx[0] is not used;
          posx[k], 1 <= k <= m+n, is the position of the variable x[k]
@@ -401,10 +367,8 @@ struct LPX
          variable xN[j], i.e. indx[m+j] = k means that posx[k] = m+j
          (if the current basis is undefined, contents of this array is
          undefined) */
-      INV *inv; /* INV inv[m,m]; */
+      INV *inv; /* INV inv[1:m,1:m]; */
       /* an invertable (factorized) form of the current basis matrix */
-      /*--------------------------------------------------------------*/
-      /* basic solution components */
       double *bbar; /* double bbar[1+m_max]; */
       /* bbar[0] is not used;
          bbar[i], 1 <= i <= m, is a value of basic variable xB[i] */
@@ -417,6 +381,17 @@ struct LPX
          cbar[j], 1 <= j <= n, is a reduced cost of non-basic variable
          xN[j] */
       /*--------------------------------------------------------------*/
+      /* mixed integer programming (MIP) segment */
+      int mip;
+      /* if this flag is set, this problem is MIP problem; if this flag
+         is clear, this problem is pure LP problem */
+      int *kind; /* int kind[1+n_max]; */
+      /* kind[0] is not used;
+         kind[j], 1 <= j <= n, specifies the kind of the j-th structural
+         variable: */
+#define LPX_CV          160   /* continuous variable */
+#define LPX_IV          161   /* integer variable */
+      /*--------------------------------------------------------------*/
       /* control parameters and statistics */
       int msg_lev;
       /* level of messages output by the solver:
@@ -424,28 +399,38 @@ struct LPX
          1 - error messages only
          2 - normal output
          3 - full output (includes informational messages) */
-      int start;
-      /* starting option:
-         0 - use standard initial basis
-         1 - use advanced initial basis
-         2 - use current basis (warm start) */
+      int scale;
+      /* scaling option:
+         0 - no scaling
+         1 - equilibration scaling
+         2 - geometric mean scaling
+         3 - geometric mean scaling, then equilibration scaling */
+      int sc_ord;
+      /* scaling order:
+         0 - at first rows, then columns
+         1 - at first columns, then rows */
+      int sc_max;
+      /* maximal number of iterations used in geometric mean scaling */
+      double sc_eps;
+      /* stopping criterion used in geometric mean scaling */
       int dual;
-      /* simplex method option:
-         0 - use primal simplex
-         1 - use dual simplex (requires the starting basis to be dual
-             feasible) */
+      /* dual simplex option:
+         0 - do not use the dual simplex
+         1 - if the initial basic solution being primal infeasible is
+             dual feasible, use the dual simplex */
       int price;
-      /* pricing option (for both primal and dual):
+      /* pricing option (for both primal and dual simplex):
          0 - textbook pricing
          1 - steepest edge pricing */
       double relax;
-      /* bound relaxation parameter used in the primal ratio test;
-         if this parameter is zero, textbook ratio test is used;
-         if this parameter is non-zero, Harris's ratio test is used;
-         in the latter case on the first pass of the ratio test basic
-         variables are allowed to slightly violate their bounds, but
-         not more than relax * tol_bnd (thus, relax is a percentage of
-         tol_bnd); typical value is 0.10--0.15 */
+      /* relaxation parameter used in the ratio test; if it is zero,
+         the textbook ratio test is used; if it is non-zero (should be
+         positive), Harris' two-pass ratio test is used; in the latter
+         case on the first pass basic variables (in the case of primal
+         simplex) or reduced costs of non-basic variables (in the case
+         of dual simplex) are allowed to slightly violate their bounds,
+         but not more than (relax * tol_bnd) or (relax * tol_dj) (thus,
+         relax is a percentage of tol_bnd or tol_dj) */
       double tol_bnd;
       /* relative tolerance used to check if the current basic solution
          is primal feasible */
@@ -453,31 +438,91 @@ struct LPX
       /* absolute tolerance used to check if the current basic solution
          is dual feasible */
       double tol_piv;
-      /* relative tolerance used to check pivotal elements in the ratio
-         test */
+      /* relative tolerance used to choose eligible pivotal elements of
+         the simplex table in the ratio test */
+      int round;
+      /* solution rounding option:
+         0 - report all computed values and reduced costs "as is"
+         1 - round computed values of basic variables and reduced costs
+             of non-basic variables toward their bounds or toward zero,
+             if possible (i.e. if it is allowed by the tolerances) */
+      double obj_ll;
+      /* lower limit of the objective function; if on the phase II the
+         objective function reaches this limit and continues decreasing,
+         the solver stops the search */
+      double obj_ul;
+      /* upper limit of the objective function; if on the phase II the
+         objective function reaches this limit and continues increasing,
+         the solver stops the search */
       int it_lim;
-      /* simplex iteration limit (maximal number of iterations allowed
-         to the solver); negative value means no limit */
+      /* simplex iterations limit; if this value is positive, it is
+         decreased by one each time when one simplex iteration has been
+         performed, and reaching zero value signals the solver to stop
+         the search; negative value means no iterations limit */
       int it_cnt;
-      /* simplex iteration count; this count is reset to zero on entry
-         to the solver and increased by one each time when the current
-         basis is changed; thus, on exit it keeps the total number of
-         simplex iterations performed by the solver */
+      /* simplex iterations count; this count is increased by one each
+         time when one simplex iteration has been performed */
+      double tm_lim;
+      /* searching time limit, in seconds; if this value is positive,
+         it is decreased each time when one simplex iteration has been
+         performed by the amount of time spent for the iteration, and
+         reaching zero value signals the solver to stop the search;
+         negative value means no time limit */
+      int out_frq;
+      /* output frequency, in iterations; this parameter specifies how
+         frequently the solver sends information about the solution to
+         the standard output */
+      double out_dly;
+      /* output delay, in seconds; this parameter specifies how long
+         the solver should delay sending information about the solution
+         to the standard output; zero value means no delay */
 };
 
-/* exit codes returned by the routine lpx_simplex: */
-#define LPX_E_OK        70 /* no errors */
-#define LPX_E_EMPTY     71 /* empty problem (no rows/columns) */
-#define LPX_E_BADB      72 /* bad initial basis (warm start only) */
-#define LPX_E_ITLIM     73 /* iteration limit exceeded */
-#define LPX_E_SING      74 /* numerical problems with basis matrix */
+/* status codes reported by the routine lpx_get_status: */
+#define LPX_OPT         170   /* optimal */
+#define LPX_FEAS        171   /* feasible */
+#define LPX_INFEAS      172   /* infeasible */
+#define LPX_NOFEAS      173   /* no feasible */
+#define LPX_UNBND       174   /* unbounded */
+#define LPX_UNDEF       175   /* undefined */
+
+/* exit codes returned by the simplex-based solver routines: */
+#define LPX_E_OK        180   /* success */
+#define LPX_E_EMPTY     181   /* empty problem */
+#define LPX_E_BADB      182   /* invalid initial basis */
+#define LPX_E_INFEAS    183   /* infeasible initial solution */
+#define LPX_E_FAULT     184   /* unable to start the search */
+#define LPX_E_OBJLL     185   /* objective lower limit reached */
+#define LPX_E_OBJUL     186   /* objective upper limit reached */
+#define LPX_E_ITLIM     187   /* iterations limit exhausted */
+#define LPX_E_TMLIM     188   /* time limit exhausted */
+#define LPX_E_NOFEAS    189   /* no feasible solution */
+#define LPX_E_INSTAB    190   /* numerical instability */
+#define LPX_E_SING      191   /* problems with basis matrix */
+
+/* control parameter identifiers */
+#define LPX_K_MSGLEV    200   /* lp->msg_lev */
+#define LPX_K_SCALE     201   /* lp->scale */
+#define LPX_K_DUAL      202   /* lp->dual */
+#define LPX_K_PRICE     203   /* lp->price */
+#define LPX_K_RELAX     204   /* lp->relax */
+#define LPX_K_TOLBND    205   /* lp->tol_bnd */
+#define LPX_K_TOLDJ     206   /* lp->tol_dj */
+#define LPX_K_TOLPIV    207   /* lp->tol_piv */
+#define LPX_K_ROUND     208   /* lp->round */
+#define LPX_K_OBJLL     209   /* lp->obj_ll */
+#define LPX_K_OBJUL     210   /* lp->obj_ul */
+#define LPX_K_ITLIM     211   /* lp->it_lim */
+#define LPX_K_ITCNT     212   /* lp->it_cnt */
+#define LPX_K_TMLIM     213   /* lp->tm_lim */
+#define LPX_K_OUTFRQ    214   /* lp->out_frq */
+#define LPX_K_OUTDLY    215   /* lp->out_dly */
+
+/* problem creating and modifying routines ---------------------------*/
 
 LPX *lpx_create_prob(void);
 /* create LP problem object */
 
-void lpx_reset_parms(LPX *lp);
-/* reset control parameters to default values */
-
 void lpx_realloc_prob(LPX *lp, int m_max, int n_max);
 /* reallocate LP problem object */
 
@@ -505,6 +550,9 @@ void lpx_set_row_bnds(LPX *lp, int i, int typx, double lb, double ub);
 void lpx_set_col_bnds(LPX *lp, int j, int typx, double lb, double ub);
 /* set (change) column bounds */
 
+void lpx_set_col_kind(LPX *lp, int j, int kind);
+/* set (change) column kind */
+
 void lpx_set_obj_dir(LPX *lp, int dir);
 /* set (change) optimization direction */
 
@@ -517,11 +565,15 @@ void lpx_set_row_coef(LPX *lp, int i, double coef);
 void lpx_set_col_coef(LPX *lp, int j, double coef);
 /* set (change) column objective coefficient */
 
-void lpx_defrag_sva(LPX *lp);
-/* defragment the sparse vector area */
+void lpx_set_row_stat(LPX *lp, int i, int stat);
+/* set (change) row status */
 
-int lpx_enlarge_cap(LPX *lp, int k, int cap);
-/* enlarge capacity of row or column */
+void lpx_set_col_stat(LPX *lp, int j, int stat);
+/* set (change) column status */
+
+void lpx_load_mat(LPX *lp,
+      void *info, double (*mat)(void *info, int *i, int *j));
+/* load constraint matrix */
 
 void lpx_set_mat_row(LPX *lp, int i, int len, int ndx[], double val[]);
 /* set (replace) row of the constraint matrix */
@@ -538,9 +590,6 @@ void lpx_mark_row(LPX *lp, int i, int mark);
 void lpx_mark_col(LPX *lp, int j, int mark);
 /* assign mark to column */
 
-void lpx_change_ordn(LPX *lp);
-/* change ordinal numbers of rows and columns */
-
 void lpx_clear_mat(LPX *lp);
 /* clear rows and columns of the constraint matrix */
 
@@ -550,8 +599,7 @@ void lpx_del_items(LPX *lp);
 void lpx_delete_prob(LPX *lp);
 /* delete LP problem object */
 
-int lpx_simplex(LPX *lp);
-/* solve LP problem by means of the simplex method */
+/* problem querying routines -----------------------------------------*/
 
 int lpx_get_num_rows(LPX *lp);
 /* determine number of rows */
@@ -579,6 +627,9 @@ void lpx_get_col_bnds(LPX *lp, int j, int *typx, double *lb,
       double *ub);
 /* obtain column bounds */
 
+int lpx_get_col_kind(LPX *lp, int j);
+/* query column kind */
+
 int lpx_get_obj_dir(LPX *lp);
 /* determine optimization direction */
 
@@ -597,12 +648,87 @@ int lpx_get_mat_row(LPX *lp, int i, int ndx[], double val[]);
 int lpx_get_mat_col(LPX *lp, int j, int ndx[], double val[]);
 /* get column of the constraint matrix */
 
+int lpx_get_status(LPX *lp);
+/* query basic solution status */
+
 int lpx_get_prim_stat(LPX *lp);
 /* query primal status of basic solution */
 
 int lpx_get_dual_stat(LPX *lp);
 /* query dual status of basic solution */
 
+void lpx_get_row_info(LPX *lp, int i, int *tagx, double *valx,
+      double *dx);
+/* obtain row solution information */
+
+void lpx_get_col_info(LPX *lp, int j, int *tagx, double *valx,
+      double *dx);
+/* obtain column solution information */
+
+double lpx_get_obj_val(LPX *lp);
+/* obtain value of the objective function */
+
+int lpx_print_sol(LPX *lp, char *fname);
+/* write problem solution in printable format */
+
+/* control parameters and statistics routines ------------------------*/
+
+void lpx_reset_parms(LPX *lp);
+/* reset control parameters to default values */
+
+void lpx_set_int_parm(LPX *lp, int parm, int val);
+/* set (change) integer control parameter */
+
+int lpx_get_int_parm(LPX *lp, int parm);
+/* query integer control parameter */
+
+void lpx_set_real_parm(LPX *lp, int parm, double val);
+/* set (change) real control parameter */
+
+double lpx_get_real_parm(LPX *lp, int parm);
+/* query real control parameter */
+
+/* problem scaling routines ------------------------------------------*/
+
+void lpx_scale_prob(LPX *lp);
+/* scale LP problem data */
+
+void lpx_unscale_prob(LPX *lp);
+/* unscale LP problem data */
+
+/* initial basis routines --------------------------------------------*/
+
+void lpx_std_basis(LPX *lp);
+/* build standard initial basis */
+
+void lpx_adv_basis(LPX *lp);
+/* build advanced initial basis */
+
+/* simplex-based solver routines -------------------------------------*/
+
+int lpx_warm_up(LPX *lp);
+/* "warm up" the initial basis */
+
+int lpx_prim_opt(LPX *lp);
+/* find optimal solution (primal simplex) */
+
+int lpx_prim_art(LPX *lp);
+/* find primal feasible solution (primal simplex) */
+
+int lpx_dual_opt(LPX *lp);
+/* find optimal solution (dual simplex) */
+
+int lpx_simplex(LPX *lp);
+/* easy-to-use driver to the simplex method */
+
+/* simplex table routines --------------------------------------------*/
+
+int lpx_eval_tab_row(LPX *lp, int k, int ndx[], double val[]);
+/* compute row of the simplex table */
+
+int lpx_eval_tab_col(LPX *lp, int k, int ndx[], double val[]);
+/* compute column of the simplex table */
+
 #endif
 
 /* eof */
