@@ -39,7 +39,6 @@
 #define check_gvec            _glp_check_gvec
 #define check_rr              _glp_check_rr
 #define check_rsm             _glp_check_rsm
-#define crash_aa              _glp_crash_aa
 #define create_rsm            _glp_create_rsm
 #define delete_rsm            _glp_delete_rsm
 #define dual_col              _glp_dual_col
@@ -310,9 +309,6 @@ extern void scale_rsm(RSM *rsm, double R[], double S[]);
 extern int build_basis(RSM *rsm, LPI *lp);
 /* build advanced basis */
 
-extern int crash_aa(MAT *A, int tag[], PER *P, PER *Q);
-/* crash augmented constraint matrix */
-
 #endif
 
 /* eof */
