@@ -32,6 +32,7 @@
 #define spx_ftran             _glp_spx_ftran
 #define spx_btran             _glp_spx_btran
 #define spx_update            _glp_spx_update
+
 #define spx_eval_xn_j         _glp_spx_eval_xn_j
 #define spx_eval_bbar         _glp_spx_eval_bbar
 #define spx_eval_pi           _glp_spx_eval_pi
@@ -58,10 +59,6 @@
 #define spx_err_in_gvec       _glp_spx_err_in_gvec
 #define spx_update_dvec       _glp_spx_update_dvec
 #define spx_err_in_dvec       _glp_spx_err_in_dvec
-#define spx_std_basis         _glp_spx_std_basis
-#define spx_adv_basis         _glp_spx_adv_basis
-#define spx_warm_up           _glp_spx_warm_up
-#define spx_solve_lp          _glp_spx_solve_lp
 
 typedef struct SPX SPX;
 
@@ -117,9 +114,9 @@ struct SPX
       /* the current reference space (used in the projected steepest
          edge technique); the flag refsp[k], 1 <= k <= m+n, is set if
          the variable x[k] belongs to the current reference space */
-      int reset;
-      /* if the simplex iteration count reaches this value, the current
-         reference space is automatically redefined */
+      int count;
+      /* if this count (used in the projected steepest edge technique)
+         gets zero, the reference space is automatically redefined */
       double *work; /* double work[1+m+n]; */
       /* working array (used for various purposes) */
       int *orig_typx; /* orig_typx[1+m+n]; */
@@ -192,7 +189,7 @@ void spx_dual_chuzr(SPX *spx, double tol);
 int spx_dual_chuzc(SPX *spx, double relax);
 /* choose non-basic variable (dual simplex) */
 
-void spx_update_bbar(SPX *spx);
+void spx_update_bbar(SPX *spx, double *obj);
 /* update values of basic variables */
 
 void spx_update_pi(SPX *spx);
@@ -228,22 +225,6 @@ void spx_update_dvec(SPX *spx);
 double spx_err_in_dvec(SPX *spx);
 /* compute maximal absolute error in dvec */
 
-/* initial basis routines --------------------------------------------*/
-
-void spx_std_basis(LPX *lp);
-/* build standard initial basis */
-
-void spx_adv_basis(LPX *lp);
-/* build advanced initial basis */
-
-int spx_warm_up(LPX *lp);
-/* "warm up" the current basis */
-
-/* simplex method solver routines ------------------------------------*/
-
-int spx_solve_lp(LPX *lp);
-/* solve LP problem by means of the simplex method */
-
 #endif
 
 /* eof */
