@@ -38,7 +38,7 @@
 -- based on the revised simplex method is used.
 ----------------------------------------------------------------------*/
 
-static char *version = "GLPSOL -- GLPK LP/MIP Solver, Version 3.0.7";
+static char *version = "GLPSOL -- GLPK LP/MIP Solver, Version 3.0.8";
 
 static char *in_fname = NULL;
 /* name of the input text file */
@@ -62,16 +62,9 @@ static int obj_dir = 0;
    '-' - minimization
    '+' - maximization */
 
-static int option = 2;
-/* this parameter tells the solver what solution should be found:
-   0 - initial solution
-   1 - primal feasible solution
-   2 - optimal solution */
-
 static int method = 'S';
 /* the method which should be used for solving a problem:
-   'O' - simplex method, old version (based on rsm1_driver)
-   'S' - simplex method, new version (based on glp_simplex2)
+   'S' - revised simplex method
    'I' - interior point method */
 
 static char *out_fname = NULL;
@@ -82,11 +75,6 @@ static int scale = 1;
 /* if this flag is set, the solver performs automatic scaling the
    problem before solving; otherwise scaling is not used */
 
-static int use_dual = 0;
-/* if this flag is set, the solver uses the dual simplex method for
-   searching for feasible solution; otherwise the solver uses the primal
-   simplex method */
-
 static int steep = 1;
 /* if this flag is set, the solver uses the steepest edge pricing (for
    both primal and dual simplex) proposed by Goldfarb & Reid; otherwise
@@ -138,12 +126,7 @@ static void display_help(char *my_name)
          "lpm only)");
       print("   --min             minimize the objective function");
       print("   --max             maximize the objective function");
-      print("   --ini             find initial solution only");
-      print("   --any             find any feasible solution");
-      print("   --fin             find final solution (default)");
-      print("   --old-sim         use simplex method, old version");
-      print("   --simplex         use simplex method, new version (defa"
-         "ult)");
+      print("   --simplex         use simplex method (default)");
       print("   --interior        use interior point method (for pure L"
          "P only)");
       print("   -o filename, --output filename");
@@ -155,12 +138,7 @@ static void display_help(char *my_name)
          "t");
       print("   -v, --version     display program version and exit");
       print("");
-      print("Options specific to simplex method (--old-sim assumed):");
-      print("   --primal          find feasible solution using primal s"
-         "implex");
-      print("                     (default)");
-      print("   --dual            find feasible solution using dual sim"
-         "plex");
+      print("Options specific to simplex method:");
       print("   --steep           use steepest edge technique (default)"
          );
       print("   --nosteep         use standard \"textbook\" pricing");
@@ -239,14 +217,6 @@ static void process_cmdline(int argc, char *argv[])
             obj_dir = '-';
          else if (p("--max"))
             obj_dir = '+';
-         else if (p("--ini"))
-            option = 0;
-         else if (p("--any"))
-            option = 1;
-         else if (p("--fin"))
-            option = 2;
-         else if (p("--old-sim"))
-            method = 'O';
          else if (p("--simplex"))
             method = 'S';
          else if (p("--interior"))
@@ -267,10 +237,6 @@ static void process_cmdline(int argc, char *argv[])
             display_help(argv[0]);
          else if (p("-v") || p("--version"))
             display_version();
-         else if (p("--primal"))
-            use_dual = 0;
-         else if (p("--dual"))
-            use_dual = 1;
          else if (p("--steep"))
             steep = 1;
          else if (p("--nosteep"))
@@ -351,38 +317,24 @@ int main(int argc, char *argv[])
       /* solve LP/MIP problem */
       t_start = clock();
       switch (method)
-      {  case 'O':
-         case 'S':
+      {  case 'S':
             if (nomip || glp_get_num_int(lp) == 0)
-            {  if (method == 'O')
-               {  struct rsm1 parm;
-                  glp_init_rsm1(&parm);
-                  parm.what = option;
-                  parm.scale = scale;
-                  parm.dual = use_dual;
-                  parm.steep = steep;
-                  parm.relax = relax;
-                  glp_call_rsm1(lp, &parm);
-               }
-               else
-               {  struct spx2 parm;
-                  glp_init_spx2(&parm);
-                  parm.scale = scale;
-                  glp_simplex2(lp, &parm);
-               }
+            {  struct spx parm;
+               glp_init_spx(&parm);
+               parm.scale = scale;
+               parm.price = steep;
+               parm.relax = relax;
+               glp_simplex(lp, &parm);
             }
             else
-            {  struct bbm1 parm;
-               glp_init_bbm1(&parm);
-               parm.what = option;
+            {  struct bnb parm;
+               glp_init_bnb(&parm);
                parm.branch = branch;
                parm.btrack = btrack;
-               parm.meth = (method == 'O' ? 0 : 1);
                parm.scale = scale;
-               parm.dual = use_dual;
                parm.steep = steep;
                parm.relax = relax;
-               glp_call_bbm1(lp, &parm);
+               glp_integer(lp, &parm);
             }
             break;
          case 'I':
@@ -391,19 +343,7 @@ int main(int argc, char *argv[])
                   "oblem; use --simplex");
                exit(EXIT_FAILURE);
             }
-            if (option == 0)
-            {  error("Interior point solver is not able to compute init"
-                  "ial basis solution; use --simplex");
-               exit(EXIT_FAILURE);
-            }
-            /* if only primal feasible solution is required, nullify
-               objective function coefficients */
-            if (option == 1)
-            {  int n = glp_get_num_cols(lp), j;
-               for (j = 0; j <= n; j++) glp_set_obj_coef(lp, j, 0.0);
-               glp_set_obj_sense(lp, '-');
-            }
-            glp_call_ipm1(lp);
+            glp_interior(lp);
             break;
          default:
             insist(method != method);
