@@ -1,4 +1,4 @@
-/* glpapi1.c */
+/* glpapi1.c (generic api routines) */
 
 /*----------------------------------------------------------------------
 -- Copyright (C) 2000, 2001, 2002 Andrew Makhorin <mao@mai2.rcnet.ru>,
@@ -2198,32 +2198,81 @@ void glp_set_row_name(LPI *lp, int i, char *name)
 }
 
 /*----------------------------------------------------------------------
--- glp_to_lpx - convert LP problem instance to low-level object.
+-- glp_lpi_to_lpx - convert LP problem instance to low-level object.
 --
 -- *Synopsis*
 --
 -- #include "glpk.h"
--- void *glp_to_lpx(LPI *lpi, int all);
+-- LPX *glp_lpi_to_lpx(LPI *lpi, int all);
 --
 -- *Description*
 --
--- The routine glp_to_lpx converts the LP problem instance, which the
--- parameter lpi points to, to an low-level LP problem object.
+-- The routine glp_lpi_to_lpx converts the LP problem instance, which
+-- the parameter lpi points to, to a low-level LP problem object.
 --
 -- If the flag all is set, the routine also copies all symbolic names
 -- to the low-level object. Otherwise no symbolic names are copied.
 --
 -- *Returns*
 --
--- The routine glp_to_lpx returns a pointer to the created low-level
--- object. */
+-- The routine returns a pointer to the created low-level object. */
 
-void *glp_to_lpx(LPI *lpi, int all)
-{     LPX *lpx;
+struct lpi_info
+{     /* transit information passed to the routine lpi_mat */
+      LPI *lpi;
+      /* LP problem instance */
+      int j;
+      /* current column number */
+      int len;
+      /* current column length */
+      int t;
+      /* pointer to the next element in the current column list */
+      int *rn; /* int rn[1+m]; */
+      /* row indices */
+      double *aj; /* double aj[1+m]; */
+      /* numerical values of elements */
+};
+
+static double lpi_mat(void *_info, int *i, int *j)
+{     /* "reads" a next element of the constraint matrix */
+      struct lpi_info *info = _info;
+      double aij;
+read: if (info->j == 0 || info->t > info->len)
+      {  /* either it is the first call or the current column has been
+            completely read; set the first/next column */
+         info->j++;
+         if (info->j > glp_get_num_cols(info->lpi))
+         {  /* the entire matrix has been completely read */
+            *i = *j = 0; /* end of file */
+            info->j = 0; /* rewind */
+            return 0.0;
+         }
+         /* obtain the first/next column */
+         info->len =
+            glp_get_col_coef(info->lpi, info->j, info->rn, info->aj);
+         /* set the pointer to the first element of the column */
+         info->t = 1;
+         /* the column may be empty, so repeat the check */
+         goto read;
+      }
+      /* obtain the next element from the current column */
+      *i = info->rn[info->t];
+      *j = info->j;
+      aij = info->aj[info->t++];
+      /* skip the element if it has zero value */
+      if (aij == 0.0) goto read;
+      /* bring the next element to the calling program */
+      return aij;
+}
+
+LPX *glp_lpi_to_lpx(LPI *lpi, int all)
+{     /* convert LPI to LPX */
+      LPX *lpx;
       int m = glp_get_num_rows(lpi);
       int n = glp_get_num_cols(lpi);
-      int i, j, k, typx, tagx, len, *rn;
-      double lb, ub, *aj;
+      struct lpi_info info;
+      int i, j, typx, kind, dir, tagx;
+      double lb, ub;
       /* create low-level LP problem object */
       lpx = lpx_create_prob();
       lpx_add_rows(lpx, m);
@@ -2236,16 +2285,6 @@ void *glp_to_lpx(LPI *lpi, int all)
          for (j = 1; j <= n; j++)
             lpx_set_col_name(lpx, j, glp_get_col_name(lpi, j));
       }
-      /* copy row scale factors */
-      for (i = 1; i <= m; i++)
-      {  lpx->rs[i] = glp_get_row_fctr(lpi, i);
-         insist(lpx->rs[i] > 0.0);
-      }
-      /* copy column scale factors */
-      for (j = 1; j <= n; j++)
-      {  lpx->rs[m+j] = glp_get_col_fctr(lpi, j);
-         insist(lpx->rs[m+j] > 0.0);
-      }
       /* copy row bounds */
       for (i = 1; i <= m; i++)
       {  glp_get_row_bnds(lpi, i, &typx, &lb, &ub);
@@ -2272,80 +2311,144 @@ void *glp_to_lpx(LPI *lpi, int all)
          }
          lpx_set_col_bnds(lpx, j, typx, lb, ub);
       }
+      /* copy column kinds */
+      for (j = 1; j <= n; j++)
+      {  if (glp_get_col_kind(lpi, j) == 'I')
+         {  for (j = 1; j <= n; j++)
+            {  switch (glp_get_col_kind(lpi, j))
+               {  case 'C': kind = LPX_CV; break;
+                  case 'I': kind = LPX_IV; break;
+                  default: insist(lpi != lpi);
+               }
+               lpx_set_col_kind(lpx, j, kind);
+            }
+            break;
+         }
+      }
       /* copy the objective function */
       switch (glp_get_obj_sense(lpi))
-      {  case '-':
-            lpx_set_obj_dir(lpx, LPX_MIN);
-            break;
-         case '+':
-            lpx_set_obj_dir(lpx, LPX_MAX);
-            break;
-         default:
-            insist(lpi != lpi);
+      {  case '-': dir = LPX_MIN; break;
+         case '+': dir = LPX_MAX; break;
+         default: insist(lpi != lpi);
       }
+      lpx_set_obj_dir(lpx, dir);
       lpx_set_obj_c0(lpx, glp_get_obj_coef(lpi, 0));
       for (j = 1; j <= n; j++)
          lpx_set_col_coef(lpx, j, glp_get_obj_coef(lpi, j));
       /* copy the constraint matrix */
-      rn = ucalloc(1+m, sizeof(int));
-      aj = ucalloc(1+m, sizeof(double));
+      info.lpi = lpi;
+      info.j = info.len = info.t = 0;
+      info.rn = ucalloc(1+m, sizeof(int));
+      info.aj = ucalloc(1+m, sizeof(double));
+      lpx_load_mat(lpx, &info, lpi_mat);
+      ufree(info.rn);
+      ufree(info.aj);
+      /* copy row statuses */
+      for (i = 1; i <= m; i++)
+      {  glp_get_row_soln(lpi, i, &tagx, NULL, NULL);
+         switch (tagx)
+         {  case 'B': tagx = LPX_BS; break;
+            case 'L': tagx = LPX_NL; break;
+            case 'U': tagx = LPX_NU; break;
+            case 'F': tagx = LPX_NF; break;
+            case 'S': tagx = LPX_NS; break;
+            default: insist(lpi != lpi);
+         }
+         lpx_set_row_stat(lpx, i, tagx);
+      }
+      /* copy column statuses */
       for (j = 1; j <= n; j++)
-      {  len = glp_get_col_coef(lpi, j, rn, aj);
-         /* remove zero coefficients */
-         k = 1;
-         while (k <= len)
-         {  if (aj[k] == 0.0)
-            {  rn[k] = rn[len];
-               aj[k] = aj[len];
-               len--;
-            }
-            else
-               k++;
-         }
-         lpx_set_mat_col(lpx, j, len, rn, aj);
-      }
-      ufree(rn);
-      ufree(aj);
-      /* copy the basis information */
-      for (k = 1; k <= m+n; k++)
-      {  if (k <= m)
-            glp_get_row_soln(lpi, k, &tagx, NULL, NULL);
-         else
-            glp_get_col_soln(lpi, k-m, &tagx, NULL, NULL);
-         if (tagx == 'B')
-            lpx->tagx[k] = LPX_BS;
-         else
-         {  switch (lpx->typx[k])
-            {  case LPX_FR:
-                  lpx->tagx[k] = LPX_NF; break;
-               case LPX_LO:
-                  lpx->tagx[k] = LPX_NL; break;
-               case LPX_UP:
-                  lpx->tagx[k] = LPX_NU; break;
-               case LPX_DB:
-                  switch (tagx)
-                  {  case 'L':
-                        lpx->tagx[k] = LPX_NL; break;
-                     case 'U':
-                        lpx->tagx[k] = LPX_NU; break;
-                     default:
-                        if (fabs(lpx->lb[k]) <= fabs(lpx->ub[k]))
-                           lpx->tagx[k] = LPX_NL;
-                        else
-                           lpx->tagx[k] = LPX_NU;
-                        break;
-                  }
-                  break;
-               case LPX_FX:
-                  lpx->tagx[k] = LPX_NS; break;
-               default:
-                  insist(lpx->typx != lpx->typx);
-            }
+      {  glp_get_col_soln(lpi, j, &tagx, NULL, NULL);
+         switch (tagx)
+         {  case 'B': tagx = LPX_BS; break;
+            case 'L': tagx = LPX_NL; break;
+            case 'U': tagx = LPX_NU; break;
+            case 'F': tagx = LPX_NF; break;
+            case 'S': tagx = LPX_NS; break;
+            default: insist(lpi != lpi);
          }
+         lpx_set_col_stat(lpx, j, tagx);
       }
+      /* return to the calling program */
       return lpx;
 }
 
+/*----------------------------------------------------------------------
+-- glp_load_lpx_soln - load LP solution from low-level object.
+--
+-- *Synopsis*
+--
+-- #include "glpk.h"
+-- void glp_load_lpx_soln(LPI *lpi, LPX *lpx, int round);
+--
+-- *Description*
+--
+-- The routine glp_load_lpx_soln loads basic solution information from
+-- an low-level object LP object specified by the parameter lpx into an
+-- LP problem instance specified by the parameter lpi.
+--
+-- The flag round specifies the rounding option. If this flag is set,
+-- computed values of basic variables and reduced costs of non-basic
+-- variables are rounded toward their bounds or toward zero (if allowed
+-- by the tolerances). Otherwise all computed values and reduced costs
+-- are reported "as is". */
+
+void glp_load_lpx_soln(LPI *lpi, LPX *lpx, int round)
+{     int m = glp_get_num_rows(lpi);
+      int n = glp_get_num_cols(lpi);
+      int b_stat, status, k, tagx;
+      double valx, dx;
+      if (!(m == lpx_get_num_rows(lpx) && n == lpx_get_num_cols(lpx)))
+         fault("glp_load_lpx_soln: inconsistent problem dimension");
+      /* determine status of the obtained solution */
+      switch (lpx_get_status(lpx))
+      {  case LPX_OPT:
+            b_stat = 'O', status = GLP_OPT; break;
+         case LPX_FEAS:
+            b_stat = 'P', status = GLP_FEAS; break;
+         case LPX_INFEAS:
+            b_stat = 'N', status = GLP_INFEAS; break;
+         case LPX_NOFEAS:
+            b_stat = 'N', status = GLP_NOFEAS; break;
+         case LPX_UNBND:
+            b_stat = 'P', status = GLP_UNBND; break;
+         case LPX_UNDEF:
+            b_stat = '?', status = GLP_UNDEF; break;
+         default: insist(lpx != lpx);
+      }
+      glp_put_soln_info(lpi, b_stat, status, lpx_get_obj_val(lpx));
+      /* copy values and reduced costs of variables */
+      for (k = 1; k <= m+n; k++)
+      {  int old_round = lpx->round;
+         lpx->round = round;
+         if (k <= m)
+            lpx_get_row_info(lpx, k, &tagx, &valx, &dx);
+         else
+            lpx_get_col_info(lpx, k-m, &tagx, &valx, &dx);
+         lpx->round = old_round;
+         switch (tagx)
+         {  case LPX_BS:
+               tagx = 'B'; break;
+            case LPX_NL:
+               tagx = 'L'; break;
+            case LPX_NU:
+               tagx = 'U'; break;
+            case LPX_NF:
+               tagx = 'F'; break;
+            case LPX_NS:
+               tagx = 'S'; break;
+            default:
+               insist(tagx != tagx);
+         }
+         if (k <= m)
+            glp_put_row_soln(lpi, k, tagx, valx, dx);
+         else
+            glp_put_col_soln(lpi, k-m, tagx, valx, dx);
+      }
+      /* return to the calling program */
+      return;
+}
+
 /*----------------------------------------------------------------------
 -- glp_init_wmps - initialize parameter block by default values.
 --
