@@ -1,4 +1,4 @@
-/* glpapi4.c (glp_call_bbm1) */
+/* glpapi4.c (glp_integer) */
 
 /*----------------------------------------------------------------------
 -- Copyright (C) 2000, 2001, 2002 Andrew Makhorin <mao@mai2.rcnet.ru>,
@@ -31,27 +31,24 @@
 #define error print
 
 /*----------------------------------------------------------------------
--- glp_init_bbm1 - initialize parameter block by default values.
+-- glp_init_bnb - initialize parameter block by default values.
 --
 -- *Synopsis*
 --
 -- #include "glpk.h"
--- void glp_init_bbm1(struct bbm1 *parm);
+-- void glp_init_bnb(struct bnb *parm);
 --
 -- *Description*
 --
--- The routine glp_init_bbm1 initializes parameter block passed to the
+-- The routine glp_init_bnb initializes parameter block passed to the
 -- branch-and-bound method by default values. */
 
-void glp_init_bbm1(struct bbm1 *parm)
-{     parm->what = 2;
-      parm->branch = BB_DRTOM;
+void glp_init_bnb(struct bnb *parm)
+{     parm->branch = BB_DRTOM;
       parm->btrack = BB_BESTP;
       parm->tol_int = 1e-6;
       parm->tol_obj = 1e-7;
-      parm->meth = 1;
       parm->scale = 1;
-      parm->dual = 0;
       parm->steep = 1;
       parm->relax = 1;
       parm->tol_bnd = 1e-8;
@@ -63,24 +60,24 @@ void glp_init_bbm1(struct bbm1 *parm)
 }
 
 /*----------------------------------------------------------------------
--- glp_call_bbm1 - solve MIP problem using branch-and-bound method.
+-- glp_integer - solve MIP problem using branch-and-bound method.
 --
 -- *Synopsis*
 --
 -- #include "glpk.h"
--- int glp_call_bbm1(LPI *lpi, struct bbm1 *parm);
+-- int glp_integer(LPI *lpi, struct bnb *parm);
 --
 -- *Description*
 --
--- The routine glp_call_bbm1 is an interface to a MIP problem solver
--- based on the branch-and-bound method.
+-- The routine glp_integer is a MIP problem solver, which is based on
+-- the branch-and-bound method.
 --
 -- This routine obtains problem data from the problem object, which the
 -- parameter lpi points to, solves the problem, and stores the computed
 -- solution back to the problem object.
 --
 -- The parameter parm is a pointer to the parameter block used by the
--- solver. This block may be initialized by the routine glp_init_bbm1
+-- solver. This block may be initialized by the routine glp_init_bnb
 -- by standard default values. It is allowed to specify NULL, in which
 -- case standard default values are used. If the problem is not very
 -- hard, standard default values fit for most cases.
@@ -108,7 +105,7 @@ void glp_init_bbm1(struct bbm1 *parm)
 --
 -- *Returns*
 --
--- The routine glp_call_bbm1 returns one of the following codes:
+-- The routine glp_integer returns one of the following codes:
 --
 -- 0 - no errors. This case means that the solver has successfully
 --     finished solving the problem. (Note, for example, if the problem
@@ -118,142 +115,92 @@ void glp_init_bbm1(struct bbm1 *parm)
 -- 2 - numerical problems with basis matrix. This case means that the
 --     solver is not able to solve the problem. */
 
-int glp_call_bbm1(LPI *lpi, struct bbm1 *parm)
+int glp_integer(LPI *lpi, struct bnb *parm)
 {     LP *lp;
       LPSOL *sol;
-      struct bbm1 _parm;
-      struct rsm1_cp rsm_cp;
-      struct spx2 spx_cp;
+      struct bnb _parm;
+      struct spx spx_cp;
       struct bbm1_cp bbm_cp;
       int ret, bstat, status, i, j;
       /* if parameter block is not specified, create the dummy one */
       if (parm == NULL)
       {  parm = &_parm;
-         glp_init_bbm1(parm);
+         glp_init_bnb(parm);
       }
       /* check control parameters for correctness */
-      if (!(0 <= parm->what && parm->what <= 2))
-         fault("glp_call_bbm1: what = %d; invalid parameter",
-            parm->what);
       if (!(parm->branch == BB_FIRST || parm->branch == BB_LAST ||
             parm->branch == BB_DRTOM))
-         fault("glp_call_bbm1: branch = %d; invalid parameter",
+         fault("glp_integer: branch = %d; invalid parameter",
             parm->branch);
       if (!(parm->btrack == BB_FIFO || parm->btrack == BB_LIFO ||
             parm->btrack == BB_BESTP))
-         fault("glp_call_bbm1: btrack = %d; invalid parameter",
+         fault("glp_integer: btrack = %d; invalid parameter",
             parm->btrack);
       if (!(0.0 < parm->tol_int && parm->tol_int < 1.0))
-         fault("glp_call_bbm1: tol_int = %g; invalid parameter",
+         fault("glp_integer: tol_int = %g; invalid parameter",
             parm->tol_int);
       if (!(0.0 < parm->tol_obj && parm->tol_obj < 1.0))
-         fault("glp_call_bbm1: tol_obj = %g; invalid parameter",
+         fault("glp_integer: tol_obj = %g; invalid parameter",
             parm->tol_obj);
-      if (!(parm->meth == 0 || parm->meth == 1))
-         fault("glp_call_bbm1: meth = %d; invalid parameter",
-            parm->meth);
       if (!(0.0 < parm->tol_bnd && parm->tol_bnd < 1.0))
-         fault("glp_call_bbm1: tol_bnd = %g; invalid parameter",
+         fault("glp_integer: tol_bnd = %g; invalid parameter",
             parm->tol_bnd);
       if (!(0.0 < parm->tol_dj && parm->tol_dj < 1.0))
-         fault("glp_call_bbm1: tol_dj = %g; invalid parameter",
+         fault("glp_integer: tol_dj = %g; invalid parameter",
             parm->tol_dj);
       if (!(0.0 < parm->tol_piv && parm->tol_piv < 1.0))
-         fault("glp_call_bbm1: tol_piv = %g; invalid parameter",
+         fault("glp_integer: tol_piv = %g; invalid parameter",
             parm->tol_piv);
       /* the problem should be of MIP class */
       if (glp_get_num_int(lpi) == 0)
-         fault("glp_call_bbm1: problem has no integer variables");
+         fault("glp_integer: problem has no integer variables");
       /* STAGE 1: FIND AN OPTIMAL SOLUTION OF THE LP RELAXATION */
-      if (parm->meth == 0)
-      {  /* extract LP problem data from the problem object */
-         lp = extract_prob(lpi);
-         /* create LP solution block */
-         sol = create_lpsol(lp->m, lp->n);
-         /* prepare control parameters */
-         rsm_cp.what = 2; /* optimal solution is required */
-         rsm_cp.scale = parm->scale;
-         rsm_cp.dual = parm->dual;
-         rsm_cp.steep = parm->steep;
-         rsm_cp.relax = parm->relax;
-         rsm_cp.tol_bnd = parm->tol_bnd;
-         rsm_cp.tol_dj = 0.30 * parm->tol_dj;
-         rsm_cp.tol_piv = parm->tol_piv;
-         rsm_cp.iter_max = 0;
-         rsm_cp.round = parm->round;
-         /* solve LP problem using the revised simplex method */
-         ret = rsm1_driver(lp, sol, &rsm_cp);
-         /* store obtained LP solution back to the problem object */
-         switch (sol->status)
-         {  case '?':
-               bstat = '?', status = GLP_UNDEF; break;
-            case 'O':
-               bstat = 'O', status = GLP_OPT; break;
-            case 'F':
-               bstat = 'P', status = GLP_FEAS; break;
-            case 'I':
-               bstat = 'N', status = GLP_INFEAS; break;
-            case 'N':
-               bstat = 'N', status = GLP_NOFEAS; break;
-            case 'U':
-               bstat = 'P', status = GLP_UNBND; break;
-            default:
-               insist(sol->status != sol->status);
-         }
-         glp_put_soln_info(lpi, bstat, status, sol->objval);
-         for (i = 1; i <= sol->m; i++)
-            glp_put_row_soln(lpi, i, sol->tagx[i], sol->valx[i],
-               sol->dx[i]);
-         for (j = 1; j <= sol->n; j++)
-            glp_put_col_soln(lpi, j, sol->tagx[sol->m+j],
-               sol->valx[sol->m+j], sol->dx[sol->m+j]);
-      }
-      else
-      {  /* prepare control parameters */
-         glp_init_spx2(&spx_cp);
-         spx_cp.scale = parm->scale;
-         spx_cp.tol_bnd = parm->tol_bnd;
-         spx_cp.tol_dj = 0.30 * parm->tol_dj;
-         spx_cp.tol_piv = parm->tol_piv;
-         spx_cp.round = parm->round;
-         /* solve LP problem using the primal simplex method */
-         ret = glp_simplex2(lpi, &spx_cp);
-         /* extract LP problem data from the problem object */
-         lp = extract_prob(lpi);
-         /* create LP solution block */
-         sol = create_lpsol(lp->m, lp->n);
-         sol->mipsol = 0;
-         switch (glp_get_status(lpi))
-         {  case GLP_UNDEF:
-               sol->status = '?'; break;
-            case GLP_OPT:
-               sol->status = 'O'; break;
-            case GLP_FEAS:
-               sol->status = 'F'; break;
-            case GLP_INFEAS:
-               sol->status = 'I'; break;
-            case GLP_NOFEAS:
-               sol->status = 'N'; break;
-            case GLP_UNBND:
-               sol->status = 'U'; break;
-            default: insist(lpi != lpi);
-         }
-         sol->objval = glp_get_obj_val(lpi);
-         for (i = 1; i <= sol->m; i++)
-            glp_get_row_soln(lpi, i, &sol->tagx[i], &sol->valx[i],
-               &sol->dx[i]);
-         for (j = 1; j <= sol->n; j++)
-            glp_get_col_soln(lpi, j, &sol->tagx[sol->m+j],
-               &sol->valx[sol->m+j], &sol->dx[sol->m+j]);
+      /* prepare control parameters */
+      glp_init_spx(&spx_cp);
+      spx_cp.scale = parm->scale;
+      spx_cp.price = parm->steep;
+      spx_cp.relax = parm->relax;
+      spx_cp.tol_bnd = parm->tol_bnd;
+      spx_cp.tol_dj = 0.30 * parm->tol_dj;
+      spx_cp.tol_piv = parm->tol_piv;
+      spx_cp.round = parm->round;
+      /* solve LP problem using the primal simplex method */
+      ret = glp_simplex(lpi, &spx_cp);
+      /* extract LP problem data from the problem object */
+      lp = extract_prob(lpi);
+      /* create LP solution block */
+      sol = create_lpsol(lp->m, lp->n);
+      sol->mipsol = 0;
+      switch (glp_get_status(lpi))
+      {  case GLP_UNDEF:
+            sol->status = '?'; break;
+         case GLP_OPT:
+            sol->status = 'O'; break;
+         case GLP_FEAS:
+            sol->status = 'F'; break;
+         case GLP_INFEAS:
+            sol->status = 'I'; break;
+         case GLP_NOFEAS:
+            sol->status = 'N'; break;
+         case GLP_UNBND:
+            sol->status = 'U'; break;
+         default: insist(lpi != lpi);
       }
+      sol->objval = glp_get_obj_val(lpi);
+      for (i = 1; i <= sol->m; i++)
+         glp_get_row_soln(lpi, i, &sol->tagx[i], &sol->valx[i],
+            &sol->dx[i]);
+      for (j = 1; j <= sol->n; j++)
+         glp_get_col_soln(lpi, j, &sol->tagx[sol->m+j],
+            &sol->valx[sol->m+j], &sol->dx[sol->m+j]);
       /* check if the LP relaxation has been solved successfully */
       if (!(ret == 0 && sol->status == 'O'))
-      {  error("glp_call_bbm1: integer optimization not possible");
+      {  error("glp_integer: integer optimization not possible");
          goto done;
       }
       /* STAGE 2: SOLVE MIP PROBLEM USING BRANCH-AND-BOUND METHOD */
       /* prepare control parameters */
-      bbm_cp.what = parm->what;
+      bbm_cp.what = 2;
       bbm_cp.branch = parm->branch;
       bbm_cp.btrack = parm->btrack;
       bbm_cp.tol_int = parm->tol_int;
