@@ -135,8 +135,13 @@ void _insist(char *expr, char *file, int line)
 -- The routine watch returns the processor time in seconds. */
 
 double watch(void)
-{     return
-         (double)clock() / (double)CLOCKS_PER_SEC;
+{     ENV *env = get_env_ptr();
+      double value;
+      value = (double)clock() / (double)CLOCKS_PER_SEC;
+      if (value < env->last_watch - 1.0)
+         print("watch: *** stop-watch overflow ***");
+      env->last_watch = value;
+      return value;
 }
 
 /* eof */
